#include "Artist.h"

using namespace std;

Artist* Artist::instance = NULL;

Artist* Artist::Instance()
{
    return instance ? instance : (instance = new Artist());
}

Artist::Artist()
{
    window_width = DEFAULT_WINDOW_WIDTH;
    window_height = DEFAULT_WINDOW_HEIGHT;
}

Rect Artist::requestBounds(string filepath)
{
    cout << "Requesting bounds for \"" << filepath << "\" ... ";
    Rect bounds;
    bounds.w=0;
    bounds.h=0;

    map<string, pair<GLuint, Rect> > ::iterator it = textureBank.find(filepath);
    // key is already present in map return it
    if( it != textureBank.end() )
    {
        cout << "found texture.\n";
        return (*it).second.second;
    }
    cout << "unknown texture.\n";
    return bounds;
}

void Artist::setWindowWidth(int width)
{
    if(!width)
        width = 1;

    window_width = width;
}

void Artist::setWindowHeight(int height)
{
    if(!height)
        height = 1;

    window_height = height;
}

GLuint Artist::requestTexture(string filepath)
{
    int width =0;
    int height = 0;
    cout << "Requesting texture " << filepath << " ... ";
    map<string, pair<GLuint, Rect> > ::iterator it = textureBank.find(filepath);
    // key is already present in map return it
    if( it != textureBank.end() )
    {
        cout << "Texture already loaded.\n";
        return (*it).second.first;
    }
    // otherwise, make a new one and return that
    else
    {
        GLuint tex = loadTexture(filepath, width, height);
        if(tex == TEXTURE_LOAD_ERROR)
        {
            cout << "Texture load error!\n" << endl;
            return TEXTURE_LOAD_ERROR;
        }
        Rect size;
        size.w = width;
        size.h = height;
        pair<GLuint, Rect> img;
        img.first = tex;
        img.second = size;
        Artist::textureBank.insert(it, pair<string, pair<GLuint, Rect> >(filepath, img));
        return tex;
    }
}

void Artist::freeTextures()
{
    map<string, pair<GLuint, Rect> > ::iterator it = textureBank.begin();
    for(; it != textureBank.end(); it++)
    {
        glDeleteTextures( 1, &((*it).second.first));
    }
    textureBank.clear();
}

 GLuint Artist::loadTexture(const string filename, int &width, int &height)
 {
   //header for testing if it is a png
   png_byte header[8];

   //open file as binary
   FILE *fp = fopen(filename.c_str(), "rb");
   if (!fp) {
     return TEXTURE_LOAD_ERROR;
   }

   cout << " opened... ";

   //read the header
   fread(header, 1, 8, fp);

   //test if png
   int is_png = !png_sig_cmp(header, 0, 8);
   if (!is_png) {
     fclose(fp);
     return TEXTURE_LOAD_ERROR;
   }

   //create png struct
   png_structp png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL,
       NULL, NULL);
   if (!png_ptr) {
     fclose(fp);
     return (TEXTURE_LOAD_ERROR);
   }

   //create png info struct
   png_infop info_ptr = png_create_info_struct(png_ptr);
   if (!info_ptr) {
     png_destroy_read_struct(&png_ptr, (png_infopp) NULL, (png_infopp) NULL);
     fclose(fp);
     return (TEXTURE_LOAD_ERROR);
   }

   //create png info struct
   png_infop end_info = png_create_info_struct(png_ptr);
   if (!end_info) {
     png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp) NULL);
     fclose(fp);
     return (TEXTURE_LOAD_ERROR);
   }

   //png error stuff, not sure libpng man suggests this.
   if (setjmp(png_jmpbuf(png_ptr))) {
     png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
     fclose(fp);
     return (TEXTURE_LOAD_ERROR);
   }

   //init png reading
   png_init_io(png_ptr, fp);

   //let libpng know you already read the first 8 bytes
   png_set_sig_bytes(png_ptr, 8);

   // read all the info up to the image data
   png_read_info(png_ptr, info_ptr);

   //variables to pass to get info
   int bit_depth, color_type;
   png_uint_32 twidth, theight;

   // get info about png
   png_get_IHDR(png_ptr, info_ptr, &twidth, &theight, &bit_depth, &color_type, NULL, NULL, NULL);

   //update width and height based on png info
   width = twidth;
   height = theight;

   // Update the png info struct.
   png_read_update_info(png_ptr, info_ptr);

   // Row size in bytes.
   int rowbytes = png_get_rowbytes(png_ptr, info_ptr);

   // Allocate the image_data as a big block, to be given to opengl
   png_byte *image_data = new png_byte[rowbytes * height];
   if (!image_data) {
     //clean up memory and close stuff
     png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
     fclose(fp);
     return TEXTURE_LOAD_ERROR;
   }

   //row_pointers is for pointing to image_data for reading the png with libpng
   png_bytep *row_pointers = new png_bytep[height];
   if (!row_pointers) {
     //clean up memory and close stuff
     png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
     delete[] image_data;
     fclose(fp);
     return TEXTURE_LOAD_ERROR;
   }
   // set the individual row_pointers to point at the correct offsets of image_data
   for (int i = 0; i < height; ++i)
     row_pointers[height - 1 - i] = image_data + i * rowbytes;

   //read the png into image_data through row_pointers
   png_read_image(png_ptr, row_pointers);


    glEnable(GL_TEXTURE_2D);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_BLEND);

   //Now generate the OpenGL texture object
   GLuint texture;
   glGenTextures(1, &texture);
   glBindTexture(GL_TEXTURE_2D, texture);
   glTexImage2D(GL_TEXTURE_2D,0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, (GLvoid*) image_data);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

   //clean up memory and close stuff
   png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
   delete[] image_data;
   delete[] row_pointers;
   fclose(fp);

   cout << "successfully loaded.\n";

   return texture;
 }
