#include "Menu.h"

Menu::Menu(StateMachine* machine) : StateLoopIFC(machine)
{
    artist = Artist::Instance();

    bg = new Image("sprites/menu.png");
    startButton = new Button("sprites/start.png");
    quitButton = new Button("sprites/quit.png");

    if(startButton)
    {
        Rect hitbox = startButton->getHitbox();
        hitbox.w = 226;
        hitbox.h = 82;
        hitbox.x = artist->windowWidth()/2 - hitbox.w /2;
        hitbox.y = artist->windowHeight() * 0.5;

        startButton->setHitbox(hitbox);
        startButton->setOffset(-74, -78);
    }

    if(quitButton)
    {
        Rect hitbox = quitButton->getHitbox();
        hitbox.w = 172;
        hitbox.h = 92;
        hitbox.x = artist->windowWidth()/2 - hitbox.w /2;
        hitbox.y = artist->windowHeight() * 0.75;

        quitButton->setHitbox(hitbox);
        quitButton->setOffset(-64, -64);
    }
}

Menu::~Menu()
{
    if(bg)
        delete bg;
    if(startButton)
        delete startButton;
    if(quitButton)
        delete quitButton;
}

void Menu::display()
{
    if(bg)
        bg->display(offsetX, offsetY);

    if(startButton)
        startButton->display();

    if(quitButton)
        quitButton->display();
}

void Menu::update(int value)
{
    glutPostRedisplay();
}

void Menu::mouseMove(int x, int y)
{
    if(startButton)
        startButton->mouseOver(x,y);
    if(quitButton)
        quitButton->mouseOver(x,y);
}

void Menu::mouseButton(int button, int state, int x, int y)
{
    if(state != GLUT_DOWN || button != GLUT_LEFT_BUTTON)
        return;

    if(startButton && startButton->mouseOver(x,y))
        stateMachine->setGameState(NEWGAME);

    if(quitButton && quitButton->mouseOver(x,y))
        stateMachine->setGameState(QUIT);
}

void Menu::reshape(int width, int height)
{
    Rect bg_bounds = bg->getBounds();
    offsetX = artist->windowWidth()/2 - bg_bounds.w /2;
    offsetY = artist->windowHeight()/2 - bg_bounds.h /2;

    int menuWidth = artist->windowWidth()>bg_bounds.w?bg_bounds.w:artist->windowWidth();
    int menuHeight = artist->windowHeight()>bg_bounds.h?bg_bounds.h:artist->windowHeight();
    int button_x_off = artist->windowWidth()>bg_bounds.w?offsetX:0;
    int button_y_off = artist->windowHeight()>bg_bounds.h?offsetY:0;

    Rect hitbox = startButton->getHitbox();
    hitbox.x = menuWidth/2 - hitbox.w /2 + button_x_off;
    hitbox.y = menuHeight * 0.5 + button_y_off;
    startButton->setHitbox(hitbox);

    hitbox = quitButton->getHitbox();
    hitbox.x = menuWidth/2 - hitbox.w /2 + button_x_off;
    hitbox.y = menuHeight * 0.75 + button_y_off;
    quitButton->setHitbox(hitbox);
}

void Menu::assumeControl(StateLoopIFC*& currentActor)
{
    cout << "--- MENU IS ASSUMING CONRTOL OF GLUT MAIN LOOP ---\n";
    currentActor = this;
    reshape(artist->windowWidth(), artist->windowHeight());
}
